"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildCss = exports.getItemCss = void 0;
var generator_1 = __importDefault(require("@babel/generator"));
var t = __importStar(require("@babel/types"));
var css_1 = require("@compiled/css");
var utils_1 = require("@compiled/utils");
var ast_1 = require("./ast");
var constants_1 = require("./constants");
var evaluate_expression_1 = require("./evaluate-expression");
var is_compiled_1 = require("./is-compiled");
var manipulate_template_literal_1 = require("./manipulate-template-literal");
var resolve_binding_1 = require("./resolve-binding");
/**
 * Will normalize the value of a `content` CSS property to ensure it has quotations around it.
 * This is done to replicate both how Styled Components behaves,
 * while not breaking how Emotion handles it.
 */
var normalizeContentValue = function (value) {
    if (value.charAt(0) !== '"' && value.charAt(0) !== "'") {
        return "\"".concat(value, "\"");
    }
    return value;
};
/**
 * Will merge all subsequent unconditional CSS expressions together.
 *
 * Input
 *
 * ```
 * [l, u, u, l, u, s, u, u, u, l, u]
 * ```
 *
 * Output
 *
 * ```
 * [s, l, uu, l, uuuu, l, u]
 * ```
 *
 * Where: `l` is a logical item,
 *        `u` is a unconditional item,
 *        `s` is a sheet
 *
 * @param arr
 */
var mergeSubsequentUnconditionalCssItems = function (arr) {
    var items = [];
    var sheets = [];
    for (var index = 0; index < arr.length; index++) {
        var item = arr[index];
        if (item.type === 'sheet') {
            sheets.push(item);
            continue;
        }
        if (item.type === 'unconditional') {
            // We found an unconditional. Let's iterate further and merge all subsequent ones.
            for (var subsequentIndex = index + 1; subsequentIndex < arr.length; subsequentIndex++) {
                var subsequentItem = arr[subsequentIndex];
                if (subsequentItem.type === 'unconditional') {
                    item.css += subsequentItem.css;
                }
                else if (subsequentItem.type === 'sheet') {
                    sheets.push(subsequentItem);
                }
                else {
                    break;
                }
                // Update index back to the new one which we'll start from again.
                index = subsequentIndex;
            }
        }
        items.push(item);
    }
    return __spreadArray(__spreadArray([], sheets, true), items, true);
};
/**
 * Returns the item css.
 *
 * @param item
 */
var getItemCss = function (item) {
    return item.type === 'conditional'
        ? [item.consequent, item.alternate].map(exports.getItemCss).join('')
        : item.css;
};
exports.getItemCss = getItemCss;
/**
 * Returns the logical item from a conditional expression.
 *
 * @param css
 * @param node
 * @param type
 */
var getLogicalItemFromConditionalExpression = function (css, node, type) {
    var expression = node.test;
    return css.map(function (item) {
        if (item.type === 'conditional') {
            return item;
        }
        if (item.type === 'logical') {
            return __assign(__assign({}, item), { expression: t.logicalExpression(item.operator, expression, item.expression) });
        }
        /**
         * Ensure that the CSS declarations in the alternate mixin are not evaluated as
         * unconditional rules.
         * Eg ${(props) => (props.isPrimary ? primary : secondary)}; where primary is
         * the consequent node and secondary is the alternate node
         * -> primary mixin will be applied only when props.isPrimary
         * -> secondary mixin will be applied only when !props.isPrimary
         *  */
        var alternateExpression = t.unaryExpression('!', expression);
        var logicalItem = {
            type: 'logical',
            css: (0, exports.getItemCss)(item),
            expression: type === 'consequent' ? expression : alternateExpression,
            operator: '&&',
        };
        return logicalItem;
    });
};
/**
 * Recursive helper function for toCSSRule to handle conditional CssItems
 *
 * @param selector
 * @param item
 */
var toCSSRuleInternal = function (selector, item) {
    return item.type === 'conditional'
        ? __assign(__assign({}, item), { consequent: toCSSRuleInternal(selector, item.consequent), alternate: toCSSRuleInternal(selector, item.alternate) }) : __assign(__assign({}, item), { css: "".concat(selector, " { ").concat((0, exports.getItemCss)(item), " }") });
};
/**
 * Maps the css in the result to CSS rules.
 *
 * @param selector
 * @param result
 */
var toCSSRule = function (selector, result) { return (__assign(__assign({}, result), { css: result.css.map(function (item) { return toCSSRuleInternal(selector, item); }) })); };
/**
 * Recursive helper function for toCSSDeclaration to handle conditional CssItems
 *
 * @param key
 * @param items
 */
var toCSSDeclarationInternal = function (key, item) {
    if (item.type === 'sheet') {
        // Leave sheets as is
        return item;
    }
    else if (item.type === 'conditional') {
        // Handle conditional branches
        return __assign(__assign({}, item), { consequent: toCSSDeclarationInternal(key, item.consequent), alternate: toCSSDeclarationInternal(key, item.alternate) });
    }
    else {
        return __assign(__assign({}, item), { css: "".concat((0, utils_1.kebabCase)(key), ": ").concat((0, exports.getItemCss)(item), ";") });
    }
};
/**
 * Maps the css in the result to CSS declarations.
 *
 * @param key
 * @param result
 */
var toCSSDeclaration = function (key, result) { return (__assign(__assign({}, result), { css: result.css.map(function (item) { return toCSSDeclarationInternal(key, item); }) })); };
/**
 * Will return the variable declarator value node from meta's own path.
 * It will compare the variable declarator node id's name with passed node name and
 * returns its stringified value along with the node.
 *
 * Eg.
 * 1. If passed node is an Identifier with name `abcd` and somewhere in own path
 * we have any variable declarator `const abcd = obj.value`, it will return
 * stringified `obj.value` along with node `obj.value`.
 * Input: `node: const abcd = obj.value`
 * Output: `{ variableName: 'obj.value', expression: Expression // obj.value Expression  }`

 * 2. If variable declarator's value is undefined `const abcd = undefined`, it will
 * return stringified `abcd` along with undefined.
 * Input: `node: const abcd = undefined`
 * Output: `{ variableName: 'abcd', expression: undefined  }`
 *
 * @param node
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var getVariableDeclaratorValueForOwnPath = function (node, meta) {
    var _a;
    // Will give stringified code for the node. If node is a function it will stringified 'function(){}'
    var expression = node;
    var variableName = (0, generator_1.default)(node).code;
    // Traverse variable declarator. If its value is not undefined/null return it along with
    // its stringified value otherwise return undefined/null along with stringified
    // variable name.
    (_a = meta.ownPath) === null || _a === void 0 ? void 0 : _a.traverse({
        VariableDeclarator: function (path) {
            if (t.isIdentifier(node) && t.isIdentifier(path.node.id, { name: node.name })) {
                var init = path.node.init;
                expression = init;
                if (meta.context === 'keyframes') {
                    // When in the keyframes context, we want to reuse the generated CSS variables as much as possible.
                    // We will do this by using the original node, and namespacing it with the keyframe name.
                    variableName = "".concat(meta.keyframe, ":").concat(node.name);
                }
                else {
                    // Get stringified value or original variable name
                    variableName = init ? (0, generator_1.default)(init).code : node.name;
                }
                path.stop();
            }
        },
    });
    return {
        expression: expression,
        variableName: variableName,
    };
};
/**
 * Will callback if the filenames from metadata do not match,
 * meaning the next meta imported from another module and was statically evaluated.
 *
 * @param meta {Metadata} The current metadata to use for the comparison
 * @param next {Metadata} The next metadata to use for the comparison
 */
var callbackIfFileIncluded = function (meta, next) {
    if (meta.state.filename !== next.state.filename) {
        meta.state.includedFiles.push(next.state.file.loc.filename);
    }
};
/**
 * Check if an imported binding resulted in CSS variables after processing.
 *
 * Currently we throw if the found CSS has any variables found from an import
 * This is because we'd need to ensure all identifiers are added to the owning
 * file - if not done they would just error at runtime. Because this isn't a
 * required feature at the moment we're deprioritizing support for this.
 */
var assertNoImportedCssVariables = function (referenceNode, meta, resolvedBinding, buildCssResult) {
    if (resolvedBinding.source === 'import' && buildCssResult.variables.length > 0) {
        throw (0, ast_1.buildCodeFrameError)("Identifier contains values that can't be statically evaluated", referenceNode, meta.parentPath);
    }
};
/**
 * Extracts CSS data from a conditional expression node.
 * Eg. props.isPrimary && props.isBolded ? ({ color: 'blue' }) : ({ color: 'red'})
 *
 * @param node Node we're interested in extracting CSS from.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var extractConditionalExpression = function (node, meta) {
    var css = [];
    var variables = [];
    var _a = constants_1.CONDITIONAL_PATHS.map(function (path) {
        var pathNode = node[path];
        var cssOutput;
        if (t.isObjectExpression(pathNode) ||
            // Check if string or template resembles CSS `property: value`
            (t.isStringLiteral(pathNode) && pathNode.value.includes(':')) ||
            (t.isTemplateLiteral(pathNode) &&
                pathNode.quasis.some(function (quasi) { return quasi.value.raw.includes(':'); })) ||
            // CSS tagged templates are already expected to be valid declarations
            (0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(pathNode, meta.state) ||
            (0, is_compiled_1.isCompiledCSSCallExpression)(pathNode, meta.state)) {
            cssOutput = (0, exports.buildCss)(pathNode, meta);
        }
        else if (t.isIdentifier(pathNode)) {
            var resolved = (0, resolve_binding_1.resolveBinding)(pathNode.name, meta, evaluate_expression_1.evaluateExpression);
            if (resolved &&
                t.isExpression(resolved.node) &&
                ((0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(resolved.node, resolved.meta.state) ||
                    (0, is_compiled_1.isCompiledCSSCallExpression)(resolved.node, resolved.meta.state))) {
                cssOutput = (0, exports.buildCss)(resolved.node, resolved.meta);
                assertNoImportedCssVariables(pathNode, meta, resolved, cssOutput);
            }
        }
        else if (t.isConditionalExpression(pathNode)) {
            cssOutput = extractConditionalExpression(pathNode, meta);
        }
        if (cssOutput) {
            variables.push.apply(variables, cssOutput.variables);
            var mergedOutput = mergeSubsequentUnconditionalCssItems(cssOutput.css);
            if (mergedOutput.length > 1) {
                // Each branch should evaluate down to a single logical or unconditional CSS Item.
                throw (0, ast_1.buildCodeFrameError)('Conditional branch contains unexpected expression', node, meta.parentPath);
            }
            return mergedOutput[0];
        }
        return undefined;
    }), consequentCss = _a[0], alternateCss = _a[1];
    if (consequentCss && alternateCss) {
        css.push({
            type: 'conditional',
            test: node.test,
            consequent: consequentCss,
            alternate: alternateCss,
        });
    }
    else if (consequentCss) {
        // convert single-sided conditional into logical statements
        css.push.apply(css, getLogicalItemFromConditionalExpression([consequentCss], node, 'consequent'));
    }
    else if (alternateCss) {
        css.push.apply(css, getLogicalItemFromConditionalExpression([alternateCss], node, 'alternate'));
    }
    return { css: css, variables: variables };
};
/**
 * Extracts CSS data from a logical expression node.
 *
 * @param node Node we're interested in extracting CSS from.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var extractLogicalExpression = function (node, meta) {
    var variables = [];
    var css = [];
    if (t.isExpression(node.body)) {
        var _a = (0, evaluate_expression_1.evaluateExpression)(node.body, meta), propValue = _a.value, updatedMeta = _a.meta;
        var result = (0, exports.buildCss)(propValue, updatedMeta);
        callbackIfFileIncluded(meta, updatedMeta);
        css.push.apply(css, result.css);
        variables.push.apply(variables, result.variables);
    }
    return { css: mergeSubsequentUnconditionalCssItems(css), variables: variables };
};
/*
 * Extracts the keyframes CSS from the `@compiled/react` keyframes usage.
 *
 * @param expression {t.CallExpression | t.TaggedTemplateExpression} The keyframes declaration
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 * @returns {CSSOutput} The keyframes CSS
 */
var extractKeyframes = function (expression, meta) {
    var prefix = meta.prefix, suffix = meta.suffix;
    // Keyframes cannot start with a number, so let's prefix it with a character
    var name = "k".concat((0, utils_1.hash)((0, generator_1.default)(expression).code));
    var selector = "@keyframes ".concat(name);
    var _a = toCSSRule(selector, (0, exports.buildCss)(t.isCallExpression(expression) ? expression.arguments : expression.quasi, __assign(__assign({}, meta), { context: 'keyframes', keyframe: name }))), css = _a.css, variables = _a.variables;
    var unexpectedCss = css.filter(function (item) { return item.type !== 'unconditional'; });
    if (unexpectedCss.length) {
        throw (0, ast_1.buildCodeFrameError)('Keyframes contains unexpected CSS', expression, meta.parentPath);
    }
    return {
        css: [
            { type: 'sheet', css: css.map(function (item) { return (0, exports.getItemCss)(item); }).join('') },
            // Use the name of the keyframe instead of the identifier
            { type: 'unconditional', css: prefix + name + suffix },
        ],
        variables: variables,
    };
};
/**
 * Extracts CSS data from an object expression node.
 *
 * @param node Node we're interested in extracting CSS from.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var extractObjectExpression = function (node, meta) {
    var css = [];
    var variables = [];
    node.properties.forEach(function (prop) {
        if (t.isObjectProperty(prop)) {
            var key = (0, ast_1.getKey)(prop.computed ? (0, evaluate_expression_1.evaluateExpression)(prop.key, meta).value : prop.key, meta);
            // Don't use prop.value directly as it extracts constants from identifiers if needed.
            var _a = (0, evaluate_expression_1.evaluateExpression)(prop.value, meta), propValue = _a.value, updatedMeta = _a.meta;
            callbackIfFileIncluded(meta, updatedMeta);
            if (t.isStringLiteral(propValue)) {
                // We've found a string literal like: `color: 'blue'`
                css.push({
                    type: 'unconditional',
                    css: "".concat((0, utils_1.kebabCase)(key), ": ").concat(key === 'content' ? normalizeContentValue(propValue.value) : propValue.value, ";"),
                });
                return;
            }
            if (t.isNumericLiteral(propValue)) {
                // We've found a numeric literal like: `fontSize: 12`
                css.push({
                    type: 'unconditional',
                    css: "".concat((0, utils_1.kebabCase)(key), ": ").concat((0, css_1.addUnitIfNeeded)(key, propValue.value), ";"),
                });
                return;
            }
            if (t.isObjectExpression(propValue) || t.isLogicalExpression(propValue)) {
                // We've found either an object like `{}` or a logical expression `isPrimary && {}`.
                // We can handle both the same way as they end up resulting in a CSS rule.
                var result = toCSSRule(key, (0, exports.buildCss)(propValue, updatedMeta));
                css.push.apply(css, result.css);
                variables.push.apply(variables, result.variables);
                return;
            }
            if (t.isTemplateLiteral(propValue)) {
                var firstExpression = propValue.expressions[0];
                var result = void 0;
                // We've found a template literal like: "fontSize: `${(props) => props.isHeading ? 20 : 14}px`"
                if (propValue.expressions.length === 1 &&
                    t.isArrowFunctionExpression(firstExpression) &&
                    t.isConditionalExpression(firstExpression.body)) {
                    (0, manipulate_template_literal_1.recomposeTemplateLiteral)(propValue, "".concat((0, utils_1.kebabCase)(key), ":"), ';');
                    result = extractTemplateLiteral(propValue, updatedMeta);
                }
                else {
                    // We've found a template literal like: "fontSize: `${fontSize}px`"
                    result = toCSSDeclaration(key, extractTemplateLiteral(propValue, updatedMeta));
                }
                css.push.apply(css, result.css);
                variables.push.apply(variables, result.variables);
                return;
            }
            if (t.isArrowFunctionExpression(propValue)) {
                /*
                  Given statments like:
                  fontWeight: (props) => props.isBold ? 'bold': 'normal',
                  marginTop: (props) => `${props.isLast ? 5 : 10}px`,
                   
                  Convert them to:
        
                  `font-weight: ${(props) => props.isBold ? 'bold': 'normal'};`
                  `margin-top: ${(props) => props.isLast ? 5 : 10}px;`,
                */
                var body = propValue.body;
                var optimizedStatement = void 0;
                if (t.isConditionalExpression(body)) {
                    optimizedStatement = t.templateLiteral([
                        t.templateElement({ raw: '', cooked: '' }),
                        t.templateElement({ raw: '', cooked: '' }, true),
                    ], [propValue]);
                }
                else if (t.isTemplateLiteral(body)) {
                    var firstExpression = body.expressions[0];
                    if (body.expressions.length === 1 && t.isConditionalExpression(firstExpression)) {
                        optimizedStatement = t.templateLiteral(body.quasis, [propValue]);
                        // Make conditional expression the body of the arrow function
                        propValue.body = firstExpression;
                    }
                }
                if (optimizedStatement) {
                    (0, manipulate_template_literal_1.recomposeTemplateLiteral)(optimizedStatement, "".concat((0, utils_1.kebabCase)(key), ":"), ';');
                    var result = extractTemplateLiteral(optimizedStatement, updatedMeta);
                    css.push.apply(css, result.css);
                    variables.push.apply(variables, result.variables);
                    return;
                }
            }
            if ((0, is_compiled_1.isCompiledKeyframesCallExpression)(propValue, updatedMeta.state) ||
                (0, is_compiled_1.isCompiledKeyframesTaggedTemplateExpression)(propValue, updatedMeta.state)) {
                var result = extractKeyframes(propValue, __assign(__assign({}, updatedMeta), { prefix: "".concat((0, utils_1.kebabCase)(key), ": "), suffix: ';' }));
                css.push.apply(css, result.css);
                variables.push.apply(variables, result.variables);
                return;
            }
            var _b = getVariableDeclaratorValueForOwnPath(propValue, updatedMeta), expression = _b.expression, variableName = _b.variableName;
            // This is the catch all for any kind of expression.
            // We don't want to explicitly handle each expression node differently if we can avoid it!
            var name_1 = "--_".concat((0, utils_1.hash)(variableName));
            variables.push({
                name: name_1,
                expression: expression,
            });
            css.push({ type: 'unconditional', css: "".concat((0, utils_1.kebabCase)(key), ": var(").concat(name_1, ");") });
        }
        else if (t.isSpreadElement(prop)) {
            var resolvedBinding = undefined;
            if (t.isIdentifier(prop.argument)) {
                resolvedBinding = (0, resolve_binding_1.resolveBinding)(prop.argument.name, meta, evaluate_expression_1.evaluateExpression);
                if (!resolvedBinding) {
                    throw (0, ast_1.buildCodeFrameError)('Variable could not be found', prop.argument, meta.parentPath);
                }
            }
            var _c = (0, evaluate_expression_1.evaluateExpression)(prop.argument, meta), propValue = _c.value, updatedMeta = _c.meta;
            var result = (0, exports.buildCss)(propValue, updatedMeta);
            callbackIfFileIncluded(meta, updatedMeta);
            resolvedBinding && assertNoImportedCssVariables(prop.argument, meta, resolvedBinding, result);
            css.push.apply(css, result.css);
            variables.push.apply(variables, result.variables);
        }
    });
    return { css: mergeSubsequentUnconditionalCssItems(css), variables: variables };
};
/**
 * Extracts CSS data from a template literal node.
 *
 * @param node Node we're interested in extracting CSS from.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var extractTemplateLiteral = function (node, meta) {
    var css = [];
    var variables = [];
    // Quasis are the string pieces of the template literal - the parts around the interpolations
    var literalResult = node.quasis.reduce(function (acc, quasi, index) {
        var nodeExpression = node.expressions[index];
        if (!nodeExpression ||
            (t.isArrowFunctionExpression(nodeExpression) && t.isLogicalExpression(nodeExpression.body))) {
            var suffix = meta.context === 'keyframes' ? '' : ';';
            return acc + quasi.value.raw + suffix;
        }
        // If quasi is ending at a point where it's expecting a value from an expression
        // i.e color:
        var isMidStatement = (0, manipulate_template_literal_1.isQuasiMidStatement)(quasi);
        var doesExpressionHaveConditionalCss = t.isArrowFunctionExpression(nodeExpression) && t.isConditionalExpression(nodeExpression.body);
        if (isMidStatement &&
            doesExpressionHaveConditionalCss &&
            !(0, manipulate_template_literal_1.hasNestedTemplateLiteralsWithConditionalRules)(node, meta)) {
            (0, manipulate_template_literal_1.optimizeConditionalStatement)(quasi, node.quasis[index + 1], nodeExpression);
        }
        var _a = (0, evaluate_expression_1.evaluateExpression)(nodeExpression, meta), interpolation = _a.value, updatedMeta = _a.meta;
        callbackIfFileIncluded(meta, updatedMeta);
        if (t.isStringLiteral(interpolation) || t.isNumericLiteral(interpolation)) {
            // Simple case - we can immediately inline the value.
            return acc + quasi.value.raw + interpolation.value;
        }
        var doesExpressionContainCssBlock = t.isObjectExpression(interpolation) ||
            (0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(interpolation, meta.state) ||
            (0, is_compiled_1.isCompiledCSSCallExpression)(interpolation, meta.state);
        if ((!isMidStatement && doesExpressionContainCssBlock) || doesExpressionHaveConditionalCss) {
            // We found something that looks like CSS.
            var result = (0, exports.buildCss)(interpolation, updatedMeta);
            if (result.css.length) {
                // Add previous accumulative CSS first before CSS from expressions
                css.push.apply(css, __spreadArray([{ type: 'unconditional', css: acc + quasi.value.raw }], result.css, false));
                variables.push.apply(variables, result.variables);
                // Reset acc as we just added them
                return '';
            }
        }
        if ((0, is_compiled_1.isCompiledKeyframesCallExpression)(interpolation, updatedMeta.state) ||
            (0, is_compiled_1.isCompiledKeyframesTaggedTemplateExpression)(interpolation, updatedMeta.state)) {
            var _b = extractKeyframes(interpolation, __assign(__assign({}, updatedMeta), { prefix: quasi.value.raw, suffix: '' })), _c = _b.css, keyframesSheet = _c[0], unconditionalKeyframesItem = _c[1], keyframeVariables = _b.variables;
            css.push(keyframesSheet);
            variables.push.apply(variables, keyframeVariables);
            return acc + (0, exports.getItemCss)(unconditionalKeyframesItem);
        }
        var _d = getVariableDeclaratorValueForOwnPath(nodeExpression, meta), expression = _d.expression, variableName = _d.variableName;
        // Everything else is considered a catch all expression.
        // The only difficulty here is what we do around prefixes and suffixes.
        // CSS variables can't have them! So we need to move them to the inline style.
        // E.g. `font-size: ${fontSize}px` will end up needing to look like:
        // `font-size: var(--_font-size)`, with the suffix moved to inline styles
        // style={{ '--_font-size': fontSize + 'px' }}
        var nextQuasis = node.quasis[index + 1];
        var _e = (0, css_1.cssAffixInterpolation)(quasi.value.raw, nextQuasis.value.raw), before = _e[0], after = _e[1];
        // Create a different CSS var name for negative value version
        var name = "--_".concat((0, utils_1.hash)(variableName)).concat(before.variablePrefix === '-' ? before.variablePrefix : '');
        // Removes any suffixes from the next quasis.
        nextQuasis.value.raw = after.css;
        variables.push({
            name: name,
            expression: expression,
            prefix: before.variablePrefix,
            suffix: after.variableSuffix,
        });
        return acc + before.css + "var(".concat(name, ")");
    }, '');
    css.push({ type: 'unconditional', css: literalResult });
    // Deals with Conditional CSS Rules from Logical Expressions
    node.expressions.forEach(function (prop) {
        if (t.isArrowFunctionExpression(prop)) {
            if (t.isLogicalExpression(prop.body)) {
                var _a = (0, evaluate_expression_1.evaluateExpression)(prop.body, meta), propValue = _a.value, updatedMeta = _a.meta;
                var result = (0, exports.buildCss)(propValue, updatedMeta);
                callbackIfFileIncluded(meta, updatedMeta);
                css.push.apply(css, result.css);
                variables.push.apply(variables, result.variables);
            }
        }
    });
    return {
        css: mergeSubsequentUnconditionalCssItems(css),
        variables: variables,
    };
};
/**
 * Will return a CSS string and CSS variables array from an input node.
 *
 * @param node Node we're interested in extracting CSS from.
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var buildCss = function (node, meta) {
    if (t.isStringLiteral(node)) {
        return { css: [{ type: 'unconditional', css: node.value }], variables: [] };
    }
    if (t.isTSAsExpression(node)) {
        return (0, exports.buildCss)(node.expression, meta);
    }
    if (t.isTemplateLiteral(node)) {
        return extractTemplateLiteral(node, meta);
    }
    if (t.isObjectExpression(node)) {
        return extractObjectExpression(node, meta);
    }
    if (t.isArrowFunctionExpression(node) && t.isLogicalExpression(node.body)) {
        return extractLogicalExpression(node, meta);
    }
    if (t.isArrowFunctionExpression(node) && t.isConditionalExpression(node.body)) {
        return extractConditionalExpression(node.body, meta);
    }
    if (t.isIdentifier(node)) {
        var resolvedBinding = (0, resolve_binding_1.resolveBinding)(node.name, meta, evaluate_expression_1.evaluateExpression);
        if (!resolvedBinding) {
            throw (0, ast_1.buildCodeFrameError)('Variable could not be found', node, meta.parentPath);
        }
        if (!t.isExpression(resolvedBinding.node)) {
            throw (0, ast_1.buildCodeFrameError)("".concat(resolvedBinding.node.type, " isn't a supported CSS type - try using an object or string"), node, meta.parentPath);
        }
        var result = (0, exports.buildCss)(resolvedBinding.node, resolvedBinding.meta);
        assertNoImportedCssVariables(node, meta, resolvedBinding, result);
        return result;
    }
    if (t.isArrayExpression(node) || Array.isArray(node)) {
        var css_2 = [];
        var variables_1 = [];
        var elements = t.isArrayExpression(node) ? node.elements : node;
        elements.forEach(function (element) {
            if (!t.isExpression(element)) {
                throw (0, ast_1.buildCodeFrameError)("".concat(element && element.type, " isn't a supported CSS type - try using an object or string"), t.isArrayExpression(node) ? node : element, meta.parentPath);
            }
            var result = t.isConditionalExpression(element)
                ? extractConditionalExpression(element, meta)
                : (0, exports.buildCss)(element, meta);
            css_2.push.apply(css_2, result.css);
            variables_1.push.apply(variables_1, result.variables);
        });
        return {
            css: css_2,
            variables: variables_1,
        };
    }
    if (t.isLogicalExpression(node)) {
        var expression_1 = node.left;
        var result = (0, exports.buildCss)(node.right, meta);
        var css = result.css.map(function (item) {
            if (item.type === 'logical') {
                return __assign(__assign({}, item), { expression: t.logicalExpression(item.operator, expression_1, item.expression) });
            }
            var logicalItem = {
                type: 'logical',
                css: (0, exports.getItemCss)(item),
                expression: expression_1,
                operator: node.operator,
            };
            return logicalItem;
        });
        return {
            css: css,
            variables: result.variables,
        };
    }
    if ((0, is_compiled_1.isCompiledCSSTaggedTemplateExpression)(node, meta.state)) {
        return (0, exports.buildCss)(node.quasi, meta);
    }
    if ((0, is_compiled_1.isCompiledCSSCallExpression)(node, meta.state)) {
        return (0, exports.buildCss)(node.arguments[0], meta);
    }
    throw (0, ast_1.buildCodeFrameError)("".concat(node.type, " isn't a supported CSS type - try using an object or string"), node, meta.parentPath);
};
exports.buildCss = buildCss;
//# sourceMappingURL=css-builders.js.map