"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applySelectors = exports.transformCssItems = void 0;
var t = __importStar(require("@babel/types"));
var css_1 = require("@compiled/css");
var css_builders_1 = require("./css-builders");
/**
 * Splits a single item's styles into sheets and an expression that handles
 * className logic at runtime.
 *
 * @param item {CssItem}
 * @param meta {Metadata} Useful metadata that can be used during the transformation
 */
var transformCssItem = function (item, meta) {
    switch (item.type) {
        case 'conditional':
            var consequent = transformCssItem(item.consequent, meta);
            var alternate = transformCssItem(item.alternate, meta);
            var defaultExpression = t.identifier('undefined');
            return {
                sheets: __spreadArray(__spreadArray([], consequent.sheets, true), alternate.sheets, true),
                classExpression: t.conditionalExpression(item.test, consequent.classExpression || defaultExpression, alternate.classExpression || defaultExpression),
            };
        case 'logical':
            var logicalCss = (0, css_1.transformCss)((0, css_builders_1.getItemCss)(item), meta.state.opts);
            return {
                sheets: logicalCss.sheets,
                classExpression: t.logicalExpression(item.operator, item.expression, t.stringLiteral(logicalCss.classNames.join(' '))),
            };
        default:
            var css = (0, css_1.transformCss)((0, css_builders_1.getItemCss)(item), meta.state.opts);
            var className = css.classNames.join(' ');
            return {
                sheets: css.sheets,
                classExpression: className.trim() ? t.stringLiteral(className) : undefined,
            };
    }
};
/**
 * Transforms CSS output into `sheets` and a list of expressions for determining
 * the classNames for a component at runtime.
 *
 * @param cssItems {CssItem[]}
 */
var transformCssItems = function (cssItems, meta) {
    var sheets = [];
    var classNames = [];
    cssItems.forEach(function (item) {
        var result = transformCssItem(item, meta);
        sheets.push.apply(sheets, result.sheets);
        if (result.classExpression) {
            classNames.push(result.classExpression);
        }
    });
    return { sheets: sheets, classNames: classNames };
};
exports.transformCssItems = transformCssItems;
/**
 * Wraps CSS within a CssItem around selector closures. Each subsequent selector
 * in the selectors array represents a nested selector.
 *
 * @param item {CssItem}
 * @param selectors {string[]}
 */
var applySelectors = function (item, selectors) {
    if (item.type === 'conditional') {
        (0, exports.applySelectors)(item.consequent, selectors);
        (0, exports.applySelectors)(item.alternate, selectors);
    }
    else {
        item.css = "".concat(selectors.join('')).concat(item.css).concat(''.padEnd(selectors.length, '}'));
    }
};
exports.applySelectors = applySelectors;
//# sourceMappingURL=transform-css-items.js.map